import ipaddress
import socket

import dpkt
from Packet import *
from dpkt.ip import IP_PROTO_TCP, IP_PROTO_UDP, IP_PROTO_ESP, IP_PROTO_AH, IP_PROTO_ICMP


class DpktSniffer:

    def __init__(self):
        pass

    def filter_packets(self, packet_from_file, network):
        BROADCAST = ipaddress.IPv4Address("255.255.255.255")

        eth = dpkt.ethernet.Ethernet(packet_from_file)

        # non-ip packety
        if not isinstance(eth.data, dpkt.ip.IP):
            return

        ip = eth.data
        # IPv6 + LLC
        if isinstance(ip, dpkt.ip6.IP6) or isinstance(eth, dpkt.llc.LLC):
            return

        if ip.p == IP_PROTO_ICMP:
            return

        # pokud je zachyceny packet mimo zkoumanou sit, zahodime jej
        if ipaddress.IPv4Address(socket.inet_ntoa(ip.src)) not in ipaddress.IPv4Network(network) and \
                ipaddress.IPv4Address(socket.inet_ntoa(ip.dst)) not in ipaddress.IPv4Network(network):
            return

        # SNMP + CLDAP/LDAP + DNS
        try:
            protocol_ip = ip.p
            if protocol_ip == IP_PROTO_TCP:
                if ip.data.sport == 161 or ip.data.dport == 161 or ip.data.sport == 389 or ip.data.dport == 389:
                    return
                if ip.data.sport == 53 or ip.data.dport == 53:
                    return
            elif protocol_ip == IP_PROTO_UDP:
                if ip.data.sport == 161 or ip.data.dport == 161 or ip.data.sport == 389 or ip.data.dport == 389:
                    return
                if ip.data.sport == 53 or ip.data.dport == 53:
                    return
        except AttributeError:
            return None

        # LLDPDU + ARP + SNMP
        if eth.type == 0x88cc or eth.type == 0x0806:
            return None

        destination_ip_address = ipaddress.IPv4Address(ip.dst)

        if destination_ip_address.is_multicast or destination_ip_address == BROADCAST:
            return None

        temp_packet = self.build_packet_structure(packet_from_file, ip)
        if temp_packet != -2:
            return temp_packet
        return None

    def build_packet_structure(self, packet_from_file, ip):
        transport_protocol = DEFAULT_PROTOCOL
        src_port = DEFAULT_PORT
        dst_port = DEFAULT_PORT
        payload_length = DEFAULT_LENGTH
        mark = DEFAULT_MARK

        try:
            protocol_ip = ip.p
            dst_ip = socket.inet_ntoa(ip.dst)
            src_ip = socket.inet_ntoa(ip.src)
            raw_data = packet_from_file
            if protocol_ip == IP_PROTO_TCP:
                transport_protocol = "TCP"
                tcp = ip.data
                src_port = tcp.sport
                dst_port = tcp.dport
                payload_length = len(tcp.data)
                raw_data = tcp.data
            elif protocol_ip == IP_PROTO_UDP:
                transport_protocol = "UDP"
                udp = ip.data
                src_port = udp.sport
                dst_port = udp.dport
                payload_length = len(udp.data)
                raw_data = udp.data
            if protocol_ip == IP_PROTO_ESP:
                transport_protocol = "ESP"
            if protocol_ip == IP_PROTO_AH:
                transport_protocol = "AH"
            packet = make_packet(src_ip, dst_ip, protocol_ip, transport_protocol, src_port, dst_port, payload_length,
                                 mark, raw_data)

            return packet
        except AttributeError:
            return -2

    # producent
    def get_ipadresses(self, input_file, packet_queue):
        if input_file is None:
            return -2
        try:
            f = open(input_file, 'rb')
            pcap = dpkt.pcapng.Reader(f)
            while True:
                try:
                    element = pcap.next()
                except dpkt.UnpackError as e:
                    print("dpkt error")
                    continue
                except StopIteration as s:
                    break
                if element is None:
                    break
                ts, buf = element
                packet_queue.put(buf)
        except ValueError as v:
            print("ERROR: Otevření pcapng souboru se nezdařilo. \n" + str(v))
            exit(-2)
