import collections

from Marks import *
from Packet import *
from openvpn_states import OpenvpnMsgTypes, OpenvpnStates


class OpenVPNAnalyzer:

    def __init__(self):
        self.states = {}
        self.buffers = {}

        self.automaton = collections.defaultdict(lambda: collections.defaultdict(OpenvpnStates))
        self.automaton[OpenvpnStates.NULL][OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.RESET_CLIENT][
            OpenvpnMsgTypes.CONTROL_HARD_RESET_SERVER] = OpenvpnStates.RESET_SERVER
        self.automaton[OpenvpnStates.RESET_SERVER][OpenvpnMsgTypes.ACK] = OpenvpnStates.ACK
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.CONTROL] = OpenvpnStates.ACK
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.ACK] = OpenvpnStates.ACK
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.DATA] = OpenvpnStates.DATA
        self.automaton[OpenvpnStates.DATA][OpenvpnMsgTypes.DATA] = OpenvpnStates.DATA

        self.automaton[OpenvpnStates.DATA][OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1] = OpenvpnStates.SOFT_RESET
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1] = OpenvpnStates.SOFT_RESET
        self.automaton[OpenvpnStates.RESET_SERVER][OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1] = OpenvpnStates.SOFT_RESET
        self.automaton[OpenvpnStates.RESET_CLIENT][OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1] = OpenvpnStates.SOFT_RESET
        self.automaton[OpenvpnStates.SOFT_RESET][OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1] = OpenvpnStates.ACK

        self.automaton[OpenvpnStates.DATA][OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.RESET_SERVER][
            OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.RESET_CLIENT][
            OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.SOFT_RESET][OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT

    def analyze(self, packet):
        # mapa stavů

        # detekce openVPN dle bytu
        control_types = ['0x40', '0x38', '0x28', '0x20', '0x18', '0x10', '0x08']
        data_types = ['0x30', '0x48']
        last_state = False

        if get_transport_protocol_from_packet(packet) == "TCP":
            # kontrolni zpravy openVPN
            com = get_communication(packet, True)
            if not com in self.buffers:
                self.buffers[com] = bytearray()
            if len(packet.raw_data) > 2 and hex(
                    (packet.raw_data[2]) in control_types or hex(packet.raw_data[2]) in data_types or len(
                            self.buffers[com]) > 0):
                self.buffers[com].extend(packet.raw_data)
            else:
                return False
            count = 0
            while True:
                if len(self.buffers[com]) > 2:
                    length = (self.buffers[com][0] * 256) + self.buffers[com][1]
                    if len(self.buffers[com]) >= length + 2:
                        openvpn_payload = self.buffers[com][2:length]
                        self.buffers[com] = self.buffers[com][length + 2:]
                    else:
                        break
                else:
                    break
                if len(openvpn_payload) == 0:
                    return False
                temp = hex(openvpn_payload[0])
                if temp in control_types:
                    # packet.is_handshake = 0
                    set_mark_on_packet(packet, Marks.OPENVPN_CONTROL)
                    last_state = self._update_state(packet, temp)
                # data openVPN
                elif temp in data_types:
                    # packet.is_handshake = 1
                    set_mark_on_packet(packet, Marks.OPENVPN_DATA)
                    last_state = self._update_state(packet, temp)
                if last_state:
                    count += 1
            return count if len(self.buffers[com]) == 0 else 0
            # return 1 if count > 0 else 0

        elif get_transport_protocol_from_packet(packet) == "UDP":
            if len(packet.raw_data) < 1:
                return 0

            temp = hex(packet.raw_data[0])

            if len(packet.raw_data) > 0 and temp in control_types:
                # packet.is_handshake = 0
                set_mark_on_packet(packet, Marks.OPENVPN_CONTROL)
                return 1 if self._update_state(packet, temp) else 0
                # data openVPN
            elif len(packet.raw_data) > 0 and temp in data_types:
                # packet.is_handshake = 1
                set_mark_on_packet(packet, Marks.OPENVPN_DATA)
                return 1 if self._update_state(packet, temp) else 0
        return 0

    def filter_false_positives(self, ip_addresses):

        for address in list(ip_addresses[Marks.OPENVPN_DATA].keys()):
            two_adresses = address.split(";")
            if two_adresses[0] > two_adresses[1]:
                temp = two_adresses[0]
                two_adresses[0] = two_adresses[1]
                two_adresses[1] = temp
            communication = ";".join(two_adresses)
            if self.states[communication] != OpenvpnStates.DATA:
                del ip_addresses[Marks.OPENVPN_DATA][address]

        for address in list(ip_addresses[Marks.OPENVPN_CONTROL].keys()):
            two_adresses = address.split(";")
            if two_adresses[0] > two_adresses[1]:
                temp = two_adresses[0]
                two_adresses[0] = two_adresses[1]
                two_adresses[1] = temp
            communication = ";".join(two_adresses)
            if self.states[communication] != OpenvpnStates.DATA:
                del ip_addresses[Marks.OPENVPN_CONTROL][address]

    def _get_msg_type(self, opcode):
        if opcode == '0x30' or opcode == '0x48':
            return OpenvpnMsgTypes.DATA
        if opcode == '0x08' or opcode == "0x38":
            return OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT
        if opcode == '0x10' or opcode == '0x40':
            return OpenvpnMsgTypes.CONTROL_HARD_RESET_SERVER
        if opcode == '0x18':
            return OpenvpnMsgTypes.CONTROL_SOFT_RESET_V1
        if opcode == '0x20':
            return OpenvpnMsgTypes.CONTROL
        if opcode == "0x28":
            return OpenvpnMsgTypes.ACK

    def _update_state(self, packet, opcode):
        communication = get_communication(packet)
        if communication not in self.states:
            self.states[communication] = OpenvpnStates.NULL
        state = self.states[communication]
        msg_type = self._get_msg_type(opcode)
        if state in self.automaton and msg_type in self.automaton[state]:
            next_state = self.automaton[state][msg_type]
            self.states[communication] = next_state
            return True
        else:
            self.states[communication] = OpenvpnStates.NULL
            return False


def get_communication(packet, directional=False):
    src_ip = get_src_ip_from_packet(packet)
    dst_ip = get_dst_ip_from_packet(packet)
    src_port = get_src_port_from_packet(packet)
    dst_port = get_dst_port_from_packet(packet)

    if src_ip < dst_ip:
        communication = src_ip + ":" + str(src_port) + ";" + dst_ip + ":" + str(dst_port)
        if directional:
            communication += '+SD'
    else:
        communication = dst_ip + ":" + str(dst_port) + ";" + src_ip + ":" + str(src_port)
        if directional:
            communication += '+DS'
    return communication
