import Marks
from openvpn import OpenVPNAnalyzer
from openvpn_states import OpenvpnStates, OpenvpnMsgTypes
from Marks import Marks
from Packet import set_mark_on_packet, get_mark_from_packet


class OpenVPNMikrotikAnalyzer(OpenVPNAnalyzer):

    def __init__(self):
        super().__init__()
        self.automaton[OpenvpnStates.NULL][OpenvpnMsgTypes.CONTROL_HARD_RESET_SERVER] = OpenvpnStates.RESET_SERVER
        self.automaton[OpenvpnStates.RESET_SERVER][
            OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.RESET_CLIENT][OpenvpnMsgTypes.ACK] = OpenvpnStates.ACK
        self.automaton[OpenvpnStates.ACK][OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.ACK
        self.automaton[OpenvpnStates.RESET_CLIENT][
            OpenvpnMsgTypes.CONTROL_HARD_RESET_CLIENT] = OpenvpnStates.RESET_CLIENT
        self.automaton[OpenvpnStates.DATA][OpenvpnMsgTypes.CONTROL] = OpenvpnStates.RESET_CLIENT

    def analyze(self, packet):
        if super(OpenVPNMikrotikAnalyzer, self).analyze(packet):
            set_mark_on_packet(packet, Marks.OPENVPN_MK_DATA if get_mark_from_packet(
                packet) == Marks.OPENVPN_DATA else Marks.OPENVPN_MK_CONTROL)
            return 1
        return 0

    def filter_false_positives(self, ip_addresses):
        for openvpn_address in list(ip_addresses[Marks.OPENVPN_CONTROL].keys()):
            openvpn_addresses = openvpn_address.split(";")
            for mikrotik_address in list(ip_addresses[Marks.OPENVPN_MK_CONTROL].keys()):
                mikrotik_addresses = mikrotik_address.split(";")
                if (openvpn_addresses[0] == mikrotik_addresses[0]) and (openvpn_addresses[1] == mikrotik_addresses[1]):
                    del ip_addresses[Marks.OPENVPN_MK_CONTROL][openvpn_address]

        for openvpn_address in list(ip_addresses[Marks.OPENVPN_DATA].keys()):
            openvpn_addresses = openvpn_address.split(";")
            for mikrotik_address in list(ip_addresses[Marks.OPENVPN_MK_DATA].keys()):
                mikrotik_addresses = mikrotik_address.split(";")
                if openvpn_addresses[0] == mikrotik_addresses[0] and openvpn_addresses[1] == mikrotik_addresses[1]:
                    del ip_addresses[Marks.OPENVPN_MK_DATA][openvpn_address]

        for address in list(ip_addresses[Marks.OPENVPN_MK_DATA].keys()):
            two_adresses = address.split(";")
            if two_adresses[0] > two_adresses[1]:
                temp = two_adresses[0]
                two_adresses[0] = two_adresses[1]
                two_adresses[1] = temp
            communication = ";".join(two_adresses)
            if self.states[communication] != OpenvpnStates.DATA:
                del ip_addresses[Marks.OPENVPN_MK_DATA][address]

        for address in list(ip_addresses[Marks.OPENVPN_MK_CONTROL].keys()):
            two_adresses = address.split(";")
            if two_adresses[0] > two_adresses[1]:
                temp = two_adresses[0]
                two_adresses[0] = two_adresses[1]
                two_adresses[1] = temp
            communication = ";".join(two_adresses)
            if self.states[communication] != OpenvpnStates.DATA:
                del ip_addresses[Marks.OPENVPN_MK_CONTROL][address]


