import ipaddress

from scapy.compat import raw
from scapy.contrib.lldp import LLDPDU
from scapy.layers.inet import TCP, UDP, IP
from scapy.layers.inet6 import IPv6
from scapy.layers.ipsec import ESP, AH
from scapy.layers.l2 import LLC, Ether
from scapy.sendrecv import sniff

from Packet import *


class ScapySniffer:

    def __init__(self, packet_queue):
        self.packet_queue = packet_queue

    def build_packet_structure(self, packet_from_file):
        transport_protocol = DEFAULT_PROTOCOL
        src_port = DEFAULT_PORT
        dst_port = DEFAULT_PORT
        src_ip = DEFAULT_IP
        dst_ip = DEFAULT_IP
        mark = DEFAULT_MARK

        protocol_ip = packet_from_file.proto
        raw_data = None
        payload_length = 0
        if TCP in packet_from_file:
            transport_protocol = "TCP"
            raw_data = raw(packet_from_file[TCP].payload)
            payload_length = len(raw_data)
            src_port = packet_from_file[TCP].sport
            dst_port = packet_from_file[TCP].dport
        elif UDP in packet_from_file:
            transport_protocol = "UDP"
            raw_data = raw(packet_from_file[UDP].payload)
            payload_length = len(raw_data)
            src_port = packet_from_file[UDP].sport
            dst_port = packet_from_file[UDP].dport
        elif ESP in packet_from_file:
            transport_protocol = "ESP"
        elif AH in packet_from_file:
            transport_protocol = "AH"
        if IP in packet_from_file:
            src_ip = packet_from_file[IP].src
            dst_ip = packet_from_file[IP].dst
        packet = make_packet(src_ip, dst_ip, protocol_ip, transport_protocol, src_port, dst_port, payload_length, mark,
                             raw_data)
        return packet

    def filter_packets(self, packet_from_file, network):
        BROADCAST = ipaddress.IPv4Address("255.255.255.255")

        if IPv6 in packet_from_file or LLC in packet_from_file:
            return None

        # LLDPDU + ARP
        if LLDPDU in packet_from_file or (Ether in packet_from_file and packet_from_file[Ether].type == 0x0806):
            return None

        if IP in packet_from_file:
            destination_ip_address = ipaddress.IPv4Address(packet_from_file[IP].dst)
            if destination_ip_address.is_multicast or destination_ip_address == BROADCAST:
                return None

        # printCommunication(temp_packet)
        temp_packet = self.build_packet_structure(packet_from_file)
        return temp_packet

    def get_ipadresses(self, input_file, packet_queue):
        if input_file is None:
            return -2
        # odfiltrovani nepotrebnych veci
        sniff(offline=input_file, prn=lambda packet: packet_queue.put(packet), store=False)
